﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.RazorPages;
using Microsoft.EntityFrameworkCore;
using MSV_LP.Data;
using MSV_LP.Models;

namespace MSV_LP.Pages.Faults
{
    public class DetailsModel : PageModel
    {
        private readonly MSV_LP.Data.MSV_LPContext _context;

        public DetailsModel(MSV_LP.Data.MSV_LPContext context)
        {
            _context = context;
        }

        public Fault Fault { get; set; }
        public DisplayDetail DisplayDetail {get; set;}
        public string Result { get; private set; }

        public async Task<IActionResult> OnGetAsync(int? id)
        {
            if (id == null)
            {
                return NotFound();
            }

            Fault = await _context.Fault.FirstOrDefaultAsync(m => m.ID == id);
            if (Fault == null)
            {
                Result = "CHYBA! ID poruchy nenalezeno.";
                return NotFound();
            }

            DisplayDetail = GetDetail(Fault);
            if (DisplayDetail == null)
            {
                return Page();
            }

            
            return Page();
        }

        public DisplayDetail GetDetail(Fault Fault)
        {
            
            if (Fault != null)
            {
                var Log = _context.FaultLog.Find(Fault.LogID); //Najde log, kam chyba patří
                var Locomotive = Log.Locomotive;
                Console.WriteLine("Hledám lokomotivu: " + Locomotive);
                var LocomotiveClass = _context.Loco.Where(veh => veh.VehID == Locomotive).Select(veh => veh.VehClass).SingleOrDefault();
                if (LocomotiveClass != null)
                {
                    Console.WriteLine("Lokomotiva " + Locomotive + " nalezena. Řada: " + LocomotiveClass);
                }
                else
                {
                    Result = "CHYBA! Lokomotiva " + Locomotive + "v databázi vozidel nenalezena.";
                    Console.WriteLine("CHYBA! Lokomotiva " + Locomotive + "v databázi vozidel nenalezena.");
                    return null;
                }

                var RegToUse = _context.Register.Where(reg => reg.Locomotive == LocomotiveClass).Select(reg => reg).SingleOrDefault();
                if (RegToUse != null)
                {
                    Console.WriteLine("Rejstřík pro řadu " + LocomotiveClass + " nalezen. Verze: " + RegToUse.Version + " SW: " + RegToUse.Software);
                }
                else
                {
                    Result = "CHYBA! Rejstřík chyb pro řadu " + LocomotiveClass +  " nenalezen.";
                    Console.WriteLine("CHYBA! Rejstřík chyb pro řadu " + LocomotiveClass + " nenalezen.");
                    return null;
                }

                var FdToUse = _context.FaultDetail.Where(fd => fd.Register_ID == RegToUse.ID).Select(fd => fd);
                if (FdToUse == null)
                {
                    Result = "CHYBA!! ID registru řady nesouhlasí s ID registru chyby.";
                    Console.WriteLine("CHYBA!! ID registru řady nesouhlasí s ID registru chyby.");
                    return null;
                }
                else
                {
                    Console.WriteLine("Nalezeno: " + FdToUse.Count().ToString() + " záznamů");
                }

                var DesToUse = FdToUse.Where(des => des.zkratka == Fault.zkratka).Select(des => des).SingleOrDefault();

                if (DesToUse != null)
                {
                    Console.WriteLine("Chyba vyhodnocena jako " + DesToUse.nazev + ".");
                    
                }
                else
                {
                    Result = "CHYBA! Odpovídající záznam pro chybu " + Fault.zkratka + " v rejstříku nenalezen.";
                    Console.WriteLine("CHYBA! Odpovídající záznam pro chybu " + Fault.zkratka + " v rejstříku nenalezen.");
                    return null;
                }
                

                DisplayDetail DisplayDetail = new DisplayDetail { } ;
                DisplayDetail.ID_LOG = Log.ID;
                DisplayDetail.ID_Fault = Fault.ID;
                DisplayDetail.VehClass = LocomotiveClass;
                DisplayDetail.VehNo = Locomotive;
                if (DesToUse.nazev.Contains("%1"))
                {
                    Console.WriteLine("Záznam obsahuje %1");
                    string primaryDescription = DesToUse.nazev;
                    string fstr = DesToUse.zdrBit;
                    List<char> charsToRemove = new List<char>() { '[', ']', '"' };
                    foreach (char c in charsToRemove)
                    {
                        fstr = fstr.Replace(c.ToString(), String.Empty);
                    }
                    string[] pomZdrBit = fstr.Split(", ");
                    Console.WriteLine("Fstr: " + fstr);
                    try
                    {
                        Console.WriteLine("zdrojové bity: " + pomZdrBit[0] + pomZdrBit[1] + pomZdrBit[2] + pomZdrBit[3] +
                        pomZdrBit[4] + pomZdrBit[5] + pomZdrBit[6] + pomZdrBit[7]);
                    }
                    catch
                    {
                        Result = "Nepodařilo se dekódovat zdrojový bit. (array index out of bounds)";
                        Console.WriteLine("Nepodařilo se dekódovat zdrojový bit. (array index out of bounds)");
                        DisplayDetail.Description = DesToUse.nazev;
                        DisplayDetail.Trvani = Fault.trvani;
                        DisplayDetail.Rychlost = Fault.rychlost;
                        DisplayDetail.PT = Fault.PT;
                        DisplayDetail.Klasifikace = Fault.kontext;
                        return DisplayDetail;
                    }
                    string pomX = null;
                    string pomY = null;
                    var X = Fault.zdrBit.Substring(1, 1);
                    var Y = Fault.zdrBit.Substring(2, 1);
                    switch (X)
                    {
                        case "0": pomX = null; break;
                        case "1": pomX = pomZdrBit[3]; break;
                        case "2": pomX = pomZdrBit[2]; break;
                        case "3": pomX = pomZdrBit[3] + " a " + pomZdrBit[2]; break;
                        case "4": pomX = pomZdrBit[1]; break;
                        case "5": pomX = pomZdrBit[1] + " a " + pomZdrBit[3]; break;
                        case "6": pomX = pomZdrBit[1] + " a " + pomZdrBit[2]; break;
                        case "7": pomX = pomZdrBit[1] + " a " + pomZdrBit[2] + " a " + pomZdrBit[3]; break;
                        case "8": pomX = pomZdrBit[0]; break;
                        case "9": pomX = pomZdrBit[0] + " a " + pomZdrBit[3]; break;
                        case "A": pomX = pomZdrBit[0] + " a " + pomZdrBit[2]; break;
                        case "B": pomX = pomZdrBit[0] + " a " + pomZdrBit[2] + " a " + pomZdrBit[3]; break;
                        case "C": pomX = pomZdrBit[0] + " a " + pomZdrBit[1]; break;
                        case "D": pomX = pomZdrBit[0] + " a " + pomZdrBit[1] + " a " + pomZdrBit[3]; break;
                        case "E": pomX = pomZdrBit[0] + " a " + pomZdrBit[1] + " a " + pomZdrBit[2]; break;
                        case "F": pomX = pomZdrBit[0] + " a " + pomZdrBit[1] + " a " + pomZdrBit[2] + " a " + pomZdrBit[3]; break;
                    }
                    switch (Y)
                    {
                        case "0": pomY = null; break;
                        case "1": pomY = pomZdrBit[7]; break;
                        case "2": pomY = pomZdrBit[6]; break;
                        case "3": pomY = pomZdrBit[7] + " a " + pomZdrBit[6]; break;
                        case "4": pomY = pomZdrBit[5]; break;
                        case "5": pomY = pomZdrBit[5] + " a " + pomZdrBit[7]; break;
                        case "6": pomY = pomZdrBit[5] + " a " + pomZdrBit[6]; break;
                        case "7": pomY = pomZdrBit[5] + " a " + pomZdrBit[6] + " a " + pomZdrBit[7]; break;
                        case "8": pomY = pomZdrBit[4]; break;
                        case "9": pomY = pomZdrBit[4] + " a " + pomZdrBit[7]; break;
                        case "A": pomY = pomZdrBit[4] + " a " + pomZdrBit[6]; break;
                        case "B": pomY = pomZdrBit[4] + " a " + pomZdrBit[6] + " a " + pomZdrBit[7]; break;
                        case "C": pomY = pomZdrBit[4] + " a " + pomZdrBit[5]; break;
                        case "D": pomY = pomZdrBit[4] + " a " + pomZdrBit[5] + " a " + pomZdrBit[7]; break;
                        case "E": pomY = pomZdrBit[4] + " a " + pomZdrBit[5] + " a " + pomZdrBit[6]; break;
                        case "F": pomY = pomZdrBit[4] + " a " + pomZdrBit[5] + " a " + pomZdrBit[6] + " a " + pomZdrBit[7]; break;
                    }
                    Console.WriteLine("Zdrojový bit vyhodnocen jako X: " + pomX + " Y: " + pomY);
                    if (pomX != null && pomY != null)
                    {
                        primaryDescription = primaryDescription.Replace("%1", (pomX + " & " + pomY));
                    }
                    else if (pomX != null && pomY == null)
                    {
                        primaryDescription = primaryDescription.Replace("%1", pomX);
                    }
                    else if (pomX == null && pomY != null)
                    {
                        primaryDescription = primaryDescription.Replace("%1", pomY);
                    }
                    DisplayDetail.Description = primaryDescription;
                }
                else
                {
                    DisplayDetail.Description = DesToUse.nazev;
                }
                DisplayDetail.Trvani = Fault.trvani;
                DisplayDetail.Rychlost = Fault.rychlost;
                DisplayDetail.PT = Fault.PT;
                DisplayDetail.Klasifikace = DesToUse.klasifikace;

                return DisplayDetail;
            }
            else
            {
                return null;
            }
        }     
    }
}
