﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using CsvHelper;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.RazorPages;
using Microsoft.AspNetCore.Mvc.Rendering;
using Microsoft.AspNetCore.Server;
using Microsoft.EntityFrameworkCore;
using MSV_LP.Data;
using MSV_LP.Models;

namespace MSV_LP.Pages.Faults
{
    public class IndexModel : PageModel
    {
        private readonly MSV_LP.Data.MSV_LPContext _context;

        public IndexModel(MSV_LP.Data.MSV_LPContext context)
        {
            _context = context;
        }

        public IList<Fault> Fault { get; set; }

        public IList<FaultLog> FaultLog { get; set; }

        [BindProperty(SupportsGet = true)]
        public string SearchString { get; set; }

        [BindProperty(SupportsGet = true)]
        public int LogID { get; set; }


        [BindProperty(SupportsGet = true)]
        public string LocomotiveNo { get; set; }


        [BindProperty(SupportsGet = true)]
        public string FaultSearchString { get; set; }

        [BindProperty(SupportsGet = true)]
        public string abc { get; set; }


        [BindProperty(SupportsGet = true)]
        public string DateNo { get; set; }

        public SelectList LogList { get; set; }
        public SelectList locoList { get; set; }
        public SelectList faultList { get; set; }
        public SelectList dateList { get; set; }

        //public void OnPost()
        //{
        //    Console.WriteLine("A post function has been executed.");
        //}
        MemoryStream stream = new MemoryStream();
        Byte[] StreamContents = new Byte[200000];


        public async Task OnGetAsync()
        {
            IQueryable<int> LogQuery = from l in _context.FaultLog
                                           orderby l.ID
                                           select l.ID;

            IQueryable<string> LocoQuery = from m in _context.FaultLog
                                            orderby m.Locomotive
                                            select m.Locomotive;

            IQueryable<string> FaultQuery = from o in _context.Fault
                                             orderby o.zkratka
                                             select o.zkratka;

            IQueryable<string> DateQuery = from n in _context.Fault
                                           orderby n.datum.ToString().Substring(0, 10)
                                           select n.datum.ToString().Substring(0,10);

            var FaultLogs = from m in _context.FaultLog
                              select m;

            var Faults = from n in _context.Fault
                        select n;
            
            if (!string.IsNullOrEmpty(SearchString))
            {
                FaultLogs = FaultLogs.Where(s => s.Locomotive.Contains(SearchString));
            }
            
            if (LogID != 0)
            {
                FaultLogs = FaultLogs.Where(a => a.ID == LogID);
            }
            if (!string.IsNullOrEmpty(FaultSearchString))
            {
                Faults = Faults.Where(y => y.zkratka == FaultSearchString);
            }
            if (!string.IsNullOrEmpty(DateNo))
                {
                Faults = Faults.Where(z => z.datum.ToString().Substring(0,10) == DateNo);
            }
            if (!string.IsNullOrEmpty(LocomotiveNo))
            {
                FaultLogs = FaultLogs.Where(x => x.Locomotive == LocomotiveNo);
            }
            LogList = new SelectList(await LogQuery.Distinct().ToListAsync());
            faultList = new SelectList(await FaultQuery.Distinct().ToListAsync());
            locoList = new SelectList(await LocoQuery.Distinct().ToListAsync());
            dateList = new SelectList(await DateQuery.Distinct().ToListAsync());


            FaultLog = await FaultLogs.ToListAsync();
            Fault = await Faults.ToListAsync();

            var writeFile = new StreamWriter(stream, leaveOpen: true);
            var csv = new CsvWriter(writeFile, CultureInfo.InvariantCulture, true);
            csv.WriteHeader<ExportModel>();
            foreach (var Log in FaultLog)
            {
                foreach (var Fault in Fault.Where(f => f.LogID == Log.ID))
                {
                    var Entry = new ExportModel
                    {
                        Vehicle = Log.Locomotive,
                        zkratka = Fault.zkratka,
                        Date = Fault.datum.ToString(),
                        Time = Fault.cas.ToString(),
                        trvani = Fault.trvani,
                        zdrBit = Fault.zdrBit,
                        rychlost = Fault.rychlost,
                        PT = Fault.PT,
                        kontext = Fault.kontext,
                        cislo = Fault.CisloPor,
                        b0 = Fault.byte0,
                        b1 = Fault.byte1,
                        b2 = Fault.byte2,
                        b3 = Fault.byte3,
                        drahaH = Fault.drahaH,
                        drahaL = Fault.drahaL
                    };
                    csv.WriteRecord<ExportModel>(Entry);
                }
            }
            await csv.FlushAsync();
            await writeFile.FlushAsync();
            await stream.FlushAsync();

            StreamContents = stream.ToArray();

            var e = (stream.Length);
            Console.WriteLine(e);
        }

        public FileStreamResult OnPostExport()
        {
            Stream stream2 = new MemoryStream(StreamContents);
            Console.WriteLine("Export initiated.");
            var d = stream2.Length;
            Console.WriteLine(d);
            return new FileStreamResult(stream, "text/csv") { FileDownloadName = "export.csv" };
        }

    }
}
