﻿using System;
using System.ComponentModel.DataAnnotations;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.RazorPages;
using Microsoft.Extensions.Configuration;
using MSV_LP.Data;
using MSV_LP.Models;
using MSV_LP.Utilities;
using System.IO;
using System.Globalization;

namespace MSV_LP.Pages.RegisterList
{
    public class UploadModel : PageModel
    {
        private readonly MSV_LP.Data.MSV_LPContext _context;

        private readonly string[] _permittedExtensions2 = { ".csv" };

        public UploadModel(MSV_LP.Data.MSV_LPContext context)
        {
            _context = context;
        }

        [BindProperty]
        public BufferedSingleFileUploadDb FileUpload { get; set; }

        public string Result { get; private set; }
        /*
        public IActionResult OnGet()
        {
            return Page();
        }
        */
        // To protect from overposting attacks, please enable the specific properties you want to bind to, for
        // more details see https://aka.ms/RazorPagesCRUD.
        public async Task<IActionResult> OnPostAsync()
        {
            if (!ModelState.IsValid)
            {
                Result = "E1 - Chyba inicializace modelu.";
                return Page();
            }

            var formFileContent2 = await FileHelpers.ProcessFormFile<BufferedSingleFileUploadDb>(FileUpload.FormFile, ModelState, _permittedExtensions2);

            if (!ModelState.IsValid)
            {
                Result = "E2 - Zvolen nepodporovaný typ souboru.";

                return Page();
            }

            // **WARNING!**
            // In the following example, the file is saved without
            // scanning the file's contents. In most production
            // scenarios, an anti-virus/anti-malware scanner API
            // is used on the file before making the file available
            // for download or for use by other systems. 
            // For more information, see the topic that accompanies 
            // this sample.

            //zpracování souboru
            string result = System.Text.Encoding.UTF8.GetString(formFileContent2);
            StringReader reader = new StringReader(result);
            reader.ReadLine(); //skip first line
            string h = reader.ReadLine(); //second line
            string[] h1 = h.Split(';');
            string Locomotive = h1[2];
            h = reader.ReadLine(); //third line
            string[] h2 = h.Split(';');
            float Version = float.Parse(h2[2], CultureInfo.InvariantCulture.NumberFormat);
            string Software = h2[3].Substring(h2[3].Length - 5);
            DateTime UploadDateTime = DateTime.Now;

            var NewRegister = new Register
            {
                Locomotive = Locomotive,
                Version = Version,
                Software = Software,
                UploadDateTime = UploadDateTime
            };

            _context.Register.Add(NewRegister);
            await _context.SaveChangesAsync();


            reader.ReadLine(); // skip 4th line

            string line;
            while ((line = reader.ReadLine()) != null) //read till the end of file
            {
                string[] Entry = line.Split(';');
                var faultDetailsEntry = new FaultDetail
                {
                    Register_ID = NewRegister.ID,
                    cislo = uint.Parse(Entry[0]),
                    Fault_ID = Entry[1],
                    zkratka = Entry[2],
                    nazev = Entry[3],
                    zdrBit = Entry[4] + "," + Entry[5],
                    klasifikace = Entry[6],
                    DD = Entry[7]
                };
                _context.FaultDetail.Add(faultDetailsEntry);
                await _context.SaveChangesAsync();
            }

            

            return RedirectToPage("./Index");
        }
    }

    public class BufferedSingleFileUploadDb
    {
        [Required]
        [Display(Name = "File")]
        public IFormFile FormFile { get; set; }

        [Display(Name = "Note")]
        [StringLength(50, MinimumLength = 0)]
        public string Note { get; set; }
    }
}
