using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.RazorPages;
using Microsoft.EntityFrameworkCore;
using MSV_LP.Data;
using MSV_LP.Models;

namespace MSV_LP.Pages.Statistics
{
    public class StatisticsModel : PageModel
    {
        private readonly MSV_LP.Data.MSV_LPContext _context;

        public StatisticsModel(MSV_LP.Data.MSV_LPContext context)
        {
            _context = context;
        }

        public class Res_FO
        {
            public string Name { get; set; }
            public uint Count { get; set; }
        }

        public class Res_FIL
        {
            public string LocomotiveNo { get; set; }
            public uint Count { get; set; }
        }

        public class Res_RC
        {
            public string LocomotiveClass { get; set; }
            public uint Count { get; set; }
        }

        List<Res_FO> Res_FaultOccurence = new List<Res_FO>();
        List<Res_FIL> Res_Locos = new List<Res_FIL>();
        List<Res_RC> Res_Classes = new List<Res_RC>();

        public async Task<JsonResult> OnGetFaultOccurenceChartData()
        {
            var invoiceChart = new CategoryChartModel();
            invoiceChart.AmountList = new List<uint>();
            invoiceChart.CategoryList = new List<string>();
            var FaultLog = _context.FaultLog.Select(p => p);
            var Fault = _context.Fault.Select(r => r.zkratka);
            List<string> Values_all = new List<string>();
            List<string> Values_specific = new List<string>();
            uint counter1 = 0;
            uint counter2 = 0;
            foreach (var entry in FaultLog)
            {
                foreach (var value in Fault)
                {
                    Values_all.Add(value);
                    if (Values_specific.Contains(value))
                    {
                        counter1++;
                    }
                    else
                    {
                        Values_specific.Add(value);
                        counter2++;
                    }
                }
            }
            Console.WriteLine("Nalezeno: " + counter1 + " zznam celkem");
            Console.WriteLine("Nalezeno: " + counter2 + " specifickch zznam");
            foreach (var value in Values_specific)
            {
                Res_FO currentResult = new Res_FO { Name = value, Count = 0 };
                foreach (var v in Values_all)
                {
                    if (v == value)
                    {
                        currentResult.Count++;
                    }
                }
                Console.WriteLine("Porucha " + currentResult.Name + " se v seznamu vyskytuje " + currentResult.Count + "x");
                Res_FaultOccurence.Add(currentResult);
            }
            IEnumerable<Res_FO> query = Res_FaultOccurence.OrderBy(p => p.Count);
            foreach (var inv in query)
            {
                invoiceChart.AmountList.Add(inv.Count);
                invoiceChart.CategoryList.Add(inv.Name);
            }
            return new JsonResult(invoiceChart);
        }

        public async Task<JsonResult> OnGetFaultsInLocosChartData()
        {
            var FaultsInLocosChart = new CategoryChartModel();

            FaultsInLocosChart.AmountList = new List<uint>();
            FaultsInLocosChart.CategoryList = new List<string>(); 

            var FaultLogs = _context.FaultLog.Select(p => p);
            var Faults = _context.Fault.Select(r => r);
            var Locomotives = FaultLogs.Select(l => l.Locomotive);
            List<string> Locomotives_specific = new List<string>();

            foreach (var Locomotive in Locomotives)
            {
                if (!Locomotives_specific.Contains(Locomotive))
                {
                    Locomotives_specific.Add(Locomotive);
                }
            }
            foreach (var Locomotive in Locomotives_specific)
            {
                Res_FIL currentResult = new Res_FIL { LocomotiveNo = Locomotive, Count = 0 };
                foreach (var Log in FaultLogs)
                {
                    if (Log.Locomotive == Locomotive)
                    {
                        var FLTS = Faults.Where(f => f.LogID == Log.ID);
                        currentResult.Count += (uint)FLTS.Count();
                        Console.WriteLine("Lokomotiva: " + Locomotive + " LOG ID: " + Log.ID + " Poet chyb: " + currentResult.Count);
                    }
                }
                Res_Locos.Add(currentResult);
             
            }

            foreach (var inv in Res_Locos)
            {
                FaultsInLocosChart.AmountList.Add(inv.Count);
                FaultsInLocosChart.CategoryList.Add(inv.LocomotiveNo);
            }

            return new JsonResult(FaultsInLocosChart);
        }

        public async Task<JsonResult> OnGetFaultsInClassesChartData()
        {
            var FaultsInClassesChart = new CategoryChartModel();

            FaultsInClassesChart.AmountList = new List<uint>();
            FaultsInClassesChart.CategoryList = new List<string>();


            var FaultLogs = _context.FaultLog.Select(p => p);
            var Faults = _context.Fault.Select(r => r);
            var Vehicles = _context.Loco.Select(c => c);
            List<string> Classes_specific = new List<string>();

            foreach (var Class in Vehicles)
            {
                if (Classes_specific.Contains(Class.VehClass))
                {
                    ;
                }
                else
                {
                    Classes_specific.Add(Class.VehClass);
                }
            }

            Console.WriteLine("Classes in register: " + Classes_specific.Count());

            foreach (var Class in Classes_specific)
            {
                Res_RC currentResult = new Res_RC { LocomotiveClass = Class, Count = 0 };
                foreach (var Loco in Vehicles.Where(veh => veh.VehClass  == Class))
                {
                        foreach (var Log in FaultLogs.Where(log => log.Locomotive == Loco.VehID))
                        {
                            var a = Faults.Where(f => f.LogID == Log.ID);
                            currentResult.Count += (uint)a.Count();
                        }
                }
                Console.WriteLine("ada: " + currentResult.LocomotiveClass + " chyb: " + currentResult.Count);
                Res_Classes.Add(currentResult);
            }

            foreach (var inv in Res_Classes)
            {
                FaultsInClassesChart.AmountList.Add(inv.Count);
                FaultsInClassesChart.CategoryList.Add(inv.LocomotiveClass);
            }

            return new JsonResult(FaultsInClassesChart);
        }
    }
}
